@@ -3,7 +3,9 @@ pub(crate) use async_function_with_timeout::*;
 pub(crate) use async_zero_sleep::*;
 pub(crate) use blocking_http_call::*;
 pub(crate) use blocking_http_call_httpx::*;
+pub(crate) use blocking_input::*;
 pub(crate) use blocking_open_call::*;
+pub(crate) use blocking_path_methods::*;
 pub(crate) use blocking_process_invocation::*;
 pub(crate) use blocking_sleep::*;
 pub(crate) use cancel_scope_no_checkpoint::*;
@@ -15,7 +17,9 @@ mod async_function_with_timeout;
 mod async_zero_sleep;
 mod blocking_http_call;
 mod blocking_http_call_httpx;
-mod blocking_open_call;
+mod blocking_input;
+pub(crate) mod blocking_open_call;
+mod blocking_path_methods;
 mod blocking_process_invocation;
 mod blocking_sleep;
 mod cancel_scope_no_checkpoint;
