@@ -5,7 +5,6 @@ pub mod settings;
 
 #[cfg(test)]
 mod tests {
-    use std::collections::BTreeSet;
     use std::path::Path;
 
     use anyhow::Result;
@@ -13,43 +12,45 @@ mod tests {
 
     use crate::registry::Rule;
 
-    use crate::test::test_path;
-    use crate::{assert_messages, settings};
-
     use super::settings::{Convention, Settings};
+    use crate::test::test_path;
+    use crate::{assert_diagnostics, settings};
 
-    #[test_case(Rule::BlankLineAfterLastSection, Path::new("sections.py"))]
+    #[test_case(Rule::MissingBlankLineAfterLastSection, Path::new("sections.py"))]
     #[test_case(Rule::NoBlankLineAfterSection, Path::new("sections.py"))]
-    #[test_case(Rule::BlankLineAfterLastSection, Path::new("D413.py"))]
-    #[test_case(Rule::BlankLineAfterSummary, Path::new("D.py"))]
+    #[test_case(Rule::MissingBlankLineAfterLastSection, Path::new("D413.py"))]
+    #[test_case(Rule::MissingBlankLineAfterSummary, Path::new("D.py"))]
     #[test_case(Rule::NoBlankLineBeforeSection, Path::new("sections.py"))]
-    #[test_case(Rule::CapitalizeSectionName, Path::new("sections.py"))]
-    #[test_case(Rule::DashedUnderlineAfterSection, Path::new("sections.py"))]
+    #[test_case(Rule::NonCapitalizedSectionName, Path::new("sections.py"))]
+    #[test_case(Rule::MissingDashedUnderlineAfterSection, Path::new("sections.py"))]
     #[test_case(Rule::UndocumentedParam, Path::new("canonical_google_examples.py"))]
     #[test_case(Rule::UndocumentedParam, Path::new("canonical_numpy_examples.py"))]
     #[test_case(Rule::UndocumentedParam, Path::new("sections.py"))]
-    #[test_case(Rule::EndsInPeriod, Path::new("D.py"))]
-    #[test_case(Rule::EndsInPeriod, Path::new("D400.py"))]
-    #[test_case(Rule::EndsInPunctuation, Path::new("D.py"))]
-    #[test_case(Rule::FirstLineCapitalized, Path::new("D.py"))]
-    #[test_case(Rule::FirstLineCapitalized, Path::new("D403.py"))]
-    #[test_case(Rule::FitsOnOneLine, Path::new("D.py"))]
-    #[test_case(Rule::IndentWithSpaces, Path::new("D.py"))]
+    #[test_case(Rule::MissingTrailingPeriod, Path::new("D.py"))]
+    #[test_case(Rule::MissingTrailingPeriod, Path::new("D400.py"))]
+    #[test_case(Rule::MissingTrailingPeriod, Path::new("D400_415.py"))]
+    #[test_case(Rule::MissingTerminalPunctuation, Path::new("D.py"))]
+    #[test_case(Rule::MissingTerminalPunctuation, Path::new("D400_415.py"))]
+    #[test_case(Rule::FirstWordUncapitalized, Path::new("D.py"))]
+    #[test_case(Rule::FirstWordUncapitalized, Path::new("D403.py"))]
+    #[test_case(Rule::UnnecessaryMultilineDocstring, Path::new("D.py"))]
+    #[test_case(Rule::DocstringTabIndentation, Path::new("D.py"))]
     #[test_case(Rule::UndocumentedMagicMethod, Path::new("D.py"))]
     #[test_case(Rule::MultiLineSummaryFirstLine, Path::new("D.py"))]
     #[test_case(Rule::MultiLineSummarySecondLine, Path::new("D.py"))]
     #[test_case(Rule::NewLineAfterLastParagraph, Path::new("D.py"))]
-    #[test_case(Rule::NewLineAfterSectionName, Path::new("sections.py"))]
-    #[test_case(Rule::NoBlankLineAfterFunction, Path::new("D.py"))]
-    #[test_case(Rule::FitsOnOneLine, Path::new("D200.py"))]
-    #[test_case(Rule::NoBlankLineAfterFunction, Path::new("D202.py"))]
+    #[test_case(Rule::MissingNewLineAfterSectionName, Path::new("sections.py"))]
+    #[test_case(Rule::BlankLineAfterFunction, Path::new("D.py"))]
+    #[test_case(Rule::UnnecessaryMultilineDocstring, Path::new("D200.py"))]
+    #[test_case(Rule::BlankLineAfterFunction, Path::new("D202.py"))]
     #[test_case(Rule::BlankLineBeforeClass, Path::new("D.py"))]
-    #[test_case(Rule::NoBlankLineBeforeFunction, Path::new("D.py"))]
+    #[test_case(Rule::BlankLineBeforeFunction, Path::new("D.py"))]
     #[test_case(Rule::BlankLinesBetweenHeaderAndContent, Path::new("sections.py"))]
     #[test_case(Rule::BlankLinesBetweenHeaderAndContent, Path::new("sphinx.py"))]
     #[test_case(Rule::OverIndentation, Path::new("D.py"))]
     #[test_case(Rule::OverIndentation, Path::new("D208.py"))]
-    #[test_case(Rule::NoSignature, Path::new("D.py"))]
+    #[test_case(Rule::SignatureInDocstring, Path::new("D.py"))]
+    #[test_case(Rule::SignatureInDocstring, Path::new("D402.py"))]
     #[test_case(Rule::SurroundingWhitespace, Path::new("D.py"))]
     #[test_case(Rule::DocstringStartsWithThis, Path::new("D.py"))]
     #[test_case(Rule::UnderIndentation, Path::new("D.py"))]
@@ -57,8 +58,8 @@ mod tests {
     #[test_case(Rule::EmptyDocstringSection, Path::new("sections.py"))]
     #[test_case(Rule::NonImperativeMood, Path::new("D401.py"))]
     #[test_case(Rule::NoBlankLineAfterSection, Path::new("D410.py"))]
-    #[test_case(Rule::OneBlankLineAfterClass, Path::new("D.py"))]
-    #[test_case(Rule::OneBlankLineBeforeClass, Path::new("D.py"))]
+    #[test_case(Rule::IncorrectBlankLineAfterClass, Path::new("D.py"))]
+    #[test_case(Rule::IncorrectBlankLineBeforeClass, Path::new("D.py"))]
     #[test_case(Rule::UndocumentedPublicClass, Path::new("D.py"))]
     #[test_case(Rule::UndocumentedPublicFunction, Path::new("D.py"))]
     #[test_case(Rule::UndocumentedPublicInit, Path::new("D.py"))]
@@ -81,13 +82,13 @@ mod tests {
     #[test_case(Rule::UndocumentedPublicNestedClass, Path::new("D.py"))]
     #[test_case(Rule::UndocumentedPublicPackage, Path::new("D.py"))]
     #[test_case(Rule::UndocumentedPublicPackage, Path::new("D104/__init__.py"))]
-    #[test_case(Rule::SectionNameEndsInColon, Path::new("D.py"))]
-    #[test_case(Rule::SectionNotOverIndented, Path::new("sections.py"))]
-    #[test_case(Rule::SectionNotOverIndented, Path::new("D214_module.py"))]
-    #[test_case(Rule::SectionUnderlineNotOverIndented, Path::new("D215.py"))]
-    #[test_case(Rule::SectionUnderlineAfterName, Path::new("sections.py"))]
-    #[test_case(Rule::SectionUnderlineMatchesSectionLength, Path::new("sections.py"))]
-    #[test_case(Rule::SectionUnderlineNotOverIndented, Path::new("sections.py"))]
+    #[test_case(Rule::MissingSectionNameColon, Path::new("D.py"))]
+    #[test_case(Rule::OverindentedSection, Path::new("sections.py"))]
+    #[test_case(Rule::OverindentedSection, Path::new("D214_module.py"))]
+    #[test_case(Rule::OverindentedSectionUnderline, Path::new("D215.py"))]
+    #[test_case(Rule::MissingSectionUnderlineAfterName, Path::new("sections.py"))]
+    #[test_case(Rule::MismatchedSectionUnderlineLength, Path::new("sections.py"))]
+    #[test_case(Rule::OverindentedSectionUnderline, Path::new("sections.py"))]
     #[test_case(Rule::OverloadWithDocstring, Path::new("D.py"))]
     #[test_case(Rule::EscapeSequenceInDocstring, Path::new("D.py"))]
     #[test_case(Rule::EscapeSequenceInDocstring, Path::new("D301.py"))]
@@ -99,16 +100,16 @@ mod tests {
             Path::new("pydocstyle").join(path).as_path(),
             &settings::LinterSettings {
                 pydocstyle: Settings {
-                    convention: None,
-                    ignore_decorators: BTreeSet::from_iter(["functools.wraps".to_string()]),
-                    property_decorators: BTreeSet::from_iter([
-                        "gi.repository.GObject.Property".to_string()
-                    ]),
+                    ignore_decorators: ["functools.wraps".to_string()].into_iter().collect(),
+                    property_decorators: ["gi.repository.GObject.Property".to_string()]
+                        .into_iter()
+                        .collect(),
+                    ..Settings::default()
                 },
                 ..settings::LinterSettings::for_rule(rule_code)
             },
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 
@@ -118,7 +119,7 @@ mod tests {
             Path::new("pydocstyle/bom.py"),
             &settings::LinterSettings::for_rule(Rule::TripleSingleQuotes),
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -129,15 +130,24 @@ mod tests {
             &settings::LinterSettings {
                 // When inferring the convention, we'll see a few false negatives.
                 // See: https://github.com/PyCQA/pydocstyle/issues/459.
-                pydocstyle: Settings {
-                    convention: None,
-                    ignore_decorators: BTreeSet::new(),
-                    property_decorators: BTreeSet::new(),
-                },
+                pydocstyle: Settings::default(),
                 ..settings::LinterSettings::for_rule(Rule::UndocumentedParam)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
+        Ok(())
+    }
+
+    #[test]
+    fn d417_unspecified_ignore_var_parameters() -> Result<()> {
+        let diagnostics = test_path(
+            Path::new("pydocstyle/D417.py"),
+            &settings::LinterSettings {
+                pydocstyle: Settings::default(),
+                ..settings::LinterSettings::for_rule(Rule::UndocumentedParam)
+            },
+        )?;
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -149,13 +159,29 @@ mod tests {
                 // With explicit Google convention, we should flag every function.
                 pydocstyle: Settings {
                     convention: Some(Convention::Google),
-                    ignore_decorators: BTreeSet::new(),
-                    property_decorators: BTreeSet::new(),
+                    ..Settings::default()
                 },
                 ..settings::LinterSettings::for_rule(Rule::UndocumentedParam)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
+        Ok(())
+    }
+
+    #[test]
+    fn d417_google_ignore_var_parameters() -> Result<()> {
+        let diagnostics = test_path(
+            Path::new("pydocstyle/D417.py"),
+            &settings::LinterSettings {
+                pydocstyle: Settings {
+                    convention: Some(Convention::Google),
+                    ignore_var_parameters: true,
+                    ..Settings::default()
+                },
+                ..settings::LinterSettings::for_rule(Rule::UndocumentedParam)
+            },
+        )?;
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -164,16 +190,15 @@ mod tests {
         let diagnostics = test_path(
             Path::new("pydocstyle/D417.py"),
             &settings::LinterSettings {
-                // With explicit Google convention, we shouldn't flag anything.
+                // With explicit numpy convention, we shouldn't flag anything.
                 pydocstyle: Settings {
                     convention: Some(Convention::Numpy),
-                    ignore_decorators: BTreeSet::new(),
-                    property_decorators: BTreeSet::new(),
+                    ..Settings::default()
                 },
                 ..settings::LinterSettings::for_rule(Rule::UndocumentedParam)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -183,10 +208,10 @@ mod tests {
             Path::new("pydocstyle/D209_D400.py"),
             &settings::LinterSettings::for_rules([
                 Rule::NewLineAfterLastParagraph,
-                Rule::EndsInPeriod,
+                Rule::MissingTrailingPeriod,
             ]),
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -205,7 +230,7 @@ mod tests {
                 Rule::UndocumentedPublicInit,
             ]),
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 }
