@@ -13,7 +13,7 @@ use lsp_types::request::{
     WorkspaceDiagnosticRequest,
 };
 use lsp_types::{
-    DiagnosticRegistrationOptions, DiagnosticServerCapabilities,
+    ClientInfo, DiagnosticRegistrationOptions, DiagnosticServerCapabilities,
     DidChangeWatchedFilesRegistrationOptions, FileSystemWatcher, Registration, RegistrationParams,
     TextDocumentContentChangeEvent, Unregistration, UnregistrationParams, Url,
 };
@@ -69,13 +69,6 @@ pub(crate) struct Session {
     /// The projects across all workspaces.
     projects: BTreeMap<SystemPathBuf, ProjectState>,
 
-    /// The project to use for files outside any workspace. For example, if the user
-    /// opens the project `<home>/my_project` in VS code but they then opens a Python file from their Desktop.
-    /// This file isn't part of the active workspace, nor is it part of any project. But we still want
-    /// to provide some basic functionality like navigation, completions, syntax highlighting, etc.
-    /// That's what we use the default project for.
-    default_project: DefaultProject,
-
     /// Initialization options that were provided by the client during server initialization.
     initialization_options: InitializationOptions,
 
@@ -113,6 +106,9 @@ pub(crate) struct Session {
     /// Registrations is a set of LSP methods that have been dynamically registered with the
     /// client.
     registrations: HashSet<String>,
+
+    /// The name of the client (editor) that connected to this server.
+    client_name: ClientName,
 }
 
 /// LSP State for a Project
@@ -148,6 +144,7 @@ impl Session {
         workspace_urls: Vec<Url>,
         initialization_options: InitializationOptions,
         native_system: Arc<dyn System + 'static + Send + Sync + RefUnwindSafe>,
+        client_name: ClientName,
         in_test: bool,
     ) -> crate::Result<Self> {
         let index = Arc::new(Index::new());
@@ -165,7 +162,6 @@ impl Session {
             workspaces,
             deferred_messages: VecDeque::new(),
             index: Some(index),
-            default_project: DefaultProject::new(),
             initialization_options,
             global_settings: Arc::new(GlobalSettings::default()),
             projects: BTreeMap::new(),
@@ -176,6 +172,7 @@ impl Session {
             suspended_workspace_diagnostics_request: None,
             revision: 0,
             registrations: HashSet::new(),
+            client_name,
         })
     }
 
@@ -304,7 +301,7 @@ impl Session {
     /// Returns a reference to the project's [`ProjectDatabase`] in which the given `path` belongs.
     ///
     /// If the path is a system path, it will return the project database that is closest to the
-    /// given path, or the default project if no project is found for the path.
+    /// given path, or the first project if no project is found for the path.
     ///
     /// If the path is a virtual path, it will return the first project database in the session.
     pub(crate) fn project_db(&self, path: &AnySystemPath) -> &ProjectDatabase {
@@ -341,15 +338,17 @@ impl Session {
     /// Returns a reference to the project's [`ProjectState`] in which the given `path` belongs.
     ///
     /// If the path is a system path, it will return the project database that is closest to the
-    /// given path, or the default project if no project is found for the path.
+    /// given path, or the first project if no project is found for the path.
     ///
     /// If the path is a virtual path, it will return the first project database in the session.
     pub(crate) fn project_state(&self, path: &AnySystemPath) -> &ProjectState {
         match path {
             AnySystemPath::System(system_path) => {
                 self.project_state_for_path(system_path).unwrap_or_else(|| {
-                    self.default_project
-                        .get(self.index.as_ref(), &self.native_system)
+                    self.projects
+                        .values()
+                        .next()
+                        .expect("To always have at least one project")
                 })
             }
             AnySystemPath::SystemVirtual(_virtual_path) => {
@@ -373,15 +372,22 @@ impl Session {
     /// [`project_db`]: Session::project_db
     pub(crate) fn project_state_mut(&mut self, path: &AnySystemPath) -> &mut ProjectState {
         match path {
-            AnySystemPath::System(system_path) => self
-                .projects
-                .range_mut(..=system_path.to_path_buf())
-                .next_back()
-                .map(|(_, project)| project)
-                .unwrap_or_else(|| {
-                    self.default_project
-                        .get_mut(self.index.as_ref(), &self.native_system)
-                }),
+            AnySystemPath::System(system_path) => {
+                let range = ..=system_path.to_path_buf();
+
+                // Using `range` here to work around a borrow checker limitation
+                // where it can't prove that the `range_mut` call and the `self.projects.values_mut`
+                // never borrow `self.projects` mutably at the same time.
+                // https://rust-lang.github.io/rfcs/2094-nll.html#problem-case-3-conditional-control-flow-across-functions
+                if self.projects.range(range.clone()).next_back().is_some() {
+                    return self.projects.range_mut(range).next_back().unwrap().1;
+                }
+
+                // TODO: Currently, ty only supports single workspaces but we need to figure out
+                // which project to use when we support multiple projects (e.g. look for the first project
+                // with an overlapping search path?)
+                self.projects.values_mut().next().unwrap()
+            }
             AnySystemPath::SystemVirtual(_virtual_path) => {
                 // TODO: Currently, ty only supports single workspace but we need to figure out
                 // which project should this virtual path belong to when there are multiple
@@ -426,19 +432,14 @@ impl Session {
             .apply_changes(changes, overrides.as_ref())
     }
 
-    /// Returns a mutable iterator over all project databases that have been initialized to this point.
-    ///
-    /// This iterator will only yield the default project database if it has been used.
+    /// Returns a mutable iterator over all project databases.
     pub(crate) fn projects_mut(&mut self) -> impl Iterator<Item = &'_ mut ProjectDatabase> + '_ {
         self.project_states_mut().map(|project| &mut project.db)
     }
 
-    /// Returns a mutable iterator over all projects that have been initialized to this point.
-    ///
-    /// This iterator will only yield the default project if it has been used.
+    /// Returns a mutable iterator over all projects.
     pub(crate) fn project_states_mut(&mut self) -> impl Iterator<Item = &'_ mut ProjectState> + '_ {
-        let default_project = self.default_project.try_get_mut();
-        self.projects.values_mut().chain(default_project)
+        self.projects.values_mut()
     }
 
     pub(crate) fn initialize_workspaces(
@@ -536,8 +537,8 @@ impl Session {
                     );
 
                     client.show_error_message(format!(
-                        "Failed to load project for workspace {url}. \
-                        Please refer to the logs for more details.",
+                        "Failed to load project for workspace {url}. {}",
+                        self.client_name.log_guidance(),
                     ));
 
                     let db_with_default_settings = ProjectMetadata::from_options(
@@ -823,6 +824,7 @@ impl Session {
                 .unwrap_or_else(|| Arc::new(WorkspaceSettings::default())),
             position_encoding: self.position_encoding,
             document: document_handle,
+            client_name: self.client_name,
         })
     }
 
@@ -841,6 +843,7 @@ impl Session {
             in_test: self.in_test,
             resolved_client_capabilities: self.resolved_client_capabilities,
             revision: self.revision,
+            client_name: self.client_name,
         }
     }
 
@@ -980,6 +983,10 @@ impl Session {
     pub(crate) fn position_encoding(&self) -> PositionEncoding {
         self.position_encoding
     }
+
+    pub(crate) fn client_name(&self) -> ClientName {
+        self.client_name
+    }
 }
 
 /// A guard that holds the only reference to the index and allows modifying it.
@@ -1029,6 +1036,7 @@ pub(crate) struct DocumentSnapshot {
     workspace_settings: Arc<WorkspaceSettings>,
     position_encoding: PositionEncoding,
     document: DocumentHandle,
+    client_name: ClientName,
 }
 
 impl DocumentSnapshot {
@@ -1075,6 +1083,10 @@ impl DocumentSnapshot {
     pub(crate) fn notebook_or_file_path(&self) -> &AnySystemPath {
         self.document.notebook_or_file_path()
     }
+
+    pub(crate) fn client_name(&self) -> ClientName {
+        self.client_name
+    }
 }
 
 /// An immutable snapshot of the current state of [`Session`].
@@ -1085,6 +1097,7 @@ pub(crate) struct SessionSnapshot {
     resolved_client_capabilities: ResolvedClientCapabilities,
     in_test: bool,
     revision: u64,
+    client_name: ClientName,
 
     /// IMPORTANT: It's important that the databases come last, or at least,
     /// after any `Arc` that we try to extract or mutate in-place using `Arc::into_inner`
@@ -1126,6 +1139,42 @@ impl SessionSnapshot {
     pub(crate) fn revision(&self) -> u64 {
         self.revision
     }
+
+    pub(crate) fn client_name(&self) -> ClientName {
+        self.client_name
+    }
+}
+
+/// Represents the client (editor) that's connected to the language server.
+#[derive(Debug, Clone, Copy)]
+pub(crate) enum ClientName {
+    Zed,
+    Other,
+}
+
+impl From<Option<ClientInfo>> for ClientName {
+    fn from(info: Option<ClientInfo>) -> Self {
+        match info {
+            Some(info) if matches!(info.name.as_str(), "Zed") => ClientName::Zed,
+            _ => ClientName::Other,
+        }
+    }
+}
+
+impl ClientName {
+    /// Returns editor-specific guidance for finding logs.
+    ///
+    /// Different editors have different ways to access language server logs, so we provide tailored
+    /// instructions based on the connected client.
+    pub(crate) fn log_guidance(self) -> &'static str {
+        match self {
+            ClientName::Zed => {
+                "Please refer to the logs for more details \
+                    (command palette: `dev: open language server logs`)."
+            }
+            ClientName::Other => "Please refer to the logs for more details.",
+        }
+    }
 }
 
 #[derive(Debug, Default)]
@@ -1245,64 +1294,6 @@ impl Workspace {
     }
 }
 
-/// Thin wrapper around the default project database that ensures it only gets initialized
-/// when it's first accessed.
-///
-/// There are a few advantages to this:
-///
-/// 1. Salsa has a fast-path for query lookups for the first created database.
-///    We really want that to be the actual project database and not our fallback database.
-/// 2. The logs when the server starts can be confusing if it once shows it uses Python X (for the default db)
-///    but then has another log that it uses Python Y (for the actual project db).
-struct DefaultProject(std::sync::OnceLock<ProjectState>);
-
-impl DefaultProject {
-    pub(crate) fn new() -> Self {
-        DefaultProject(std::sync::OnceLock::new())
-    }
-
-    pub(crate) fn get(
-        &self,
-        index: Option<&Arc<Index>>,
-        fallback_system: &Arc<dyn System + 'static + Send + Sync + RefUnwindSafe>,
-    ) -> &ProjectState {
-        self.0.get_or_init(|| {
-            tracing::info!("Initializing the default project");
-
-            let index = index.unwrap();
-            let system = LSPSystem::new(index.clone(), fallback_system.clone());
-            let metadata = ProjectMetadata::from_options(
-                Options::default(),
-                system.current_directory().to_path_buf(),
-                None,
-                MisconfigurationMode::UseDefault,
-            )
-            .unwrap();
-
-            ProjectState {
-                db: ProjectDatabase::new(metadata, system).unwrap(),
-                untracked_files_with_pushed_diagnostics: Vec::new(),
-            }
-        })
-    }
-
-    pub(crate) fn get_mut(
-        &mut self,
-        index: Option<&Arc<Index>>,
-        fallback_system: &Arc<dyn System + 'static + Send + Sync + RefUnwindSafe>,
-    ) -> &mut ProjectState {
-        let _ = self.get(index, fallback_system);
-
-        // SAFETY: The `OnceLock` is guaranteed to be initialized at this point because
-        // we called `get` above, which initializes it if it wasn't already.
-        self.0.get_mut().unwrap()
-    }
-
-    pub(crate) fn try_get_mut(&mut self) -> Option<&mut ProjectState> {
-        self.0.get_mut()
-    }
-}
-
 /// A workspace diagnostic request that didn't yield any changes or diagnostic
 /// when it ran the last time.
 #[derive(Debug)]
